--[[
    ArenaLive [Spectator] is an user interface for spectated arena 
	wargames in World of Warcraft.
    Copyright (C) 2015  Harald Bhm <harald@boehm.agency>
	Further contributors: Jochen Taeschner and Romina Schmidt.
	
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	ADDITIONAL PERMISSION UNDER GNU GPL VERSION 3 SECTION 7:
	As a special exception, the copyright holder of this add-on gives you
	permission to link this add-on with independent proprietary software,
	regardless of the license terms of the independent proprietary software.
]]
local addonName, L = ...;
local PlayerInfoFrame = ArenaLiveSpectatorPlayerInfoFrame;
local NameText = ArenaLive:GetHandler("NameText");
local NUM_TALENTS = 7;
local NUM_PVPTALENTS = 6;

function PlayerInfoFrame:Initialise ()
	self.title:SetText(L["Player Information"]);
	self.talentFrame.title:SetText(L["Talents:"])
	self.pvpTalentFrame.title:SetText(L["PVP Talents:"]);
	
	self:RegisterForDrag("LeftButton");
	
	
	self:SetScript("OnDragStart", self.OnDragStart)
	self:SetScript("OnDragStop", self.StopMovingOrSizing)
	self:SetScript("OnShow", self.OnShow);
	
	self:Update();
end

function PlayerInfoFrame:Update ()
	if ( self.unit ) then
		local name = NameText:GetNickname(self.unit) or UnitName(self.unit);
		local specID = ArenaLiveSpectator.UnitCache:GetUnitSpecialisation(self.unit);
		if ( name and specID ) then
			if ( self.waitingOnPlayerInfo ) then
				self.waitingOnPlayerInfo = nil;
				self:Show();
			end
			
			-- Set Title Frame:
			local db = ArenaLive:GetDBComponent(addonName);
			local _, _, _, _, teamID = ArenaLiveSpectator.UnitCache:GetUnitInfo(self.unit);
			local red, green, blue, alpha;
			if ( teamID == 1 ) then	
				red, green, blue, alpha = unpack(db.TeamA.Colour);
			elseif ( teamID == 0 ) then
				red, green, blue, alpha = unpack(db.TeamB.Colour);
			else
				red, green, blue, alpha = 1, 1, 1, 1;
			end
			
			self.titleFrame.playerName:SetText(name);
			self.titleFrame.playerName:SetTextColor(red, green, blue, alpha);
			
			local _, specName, _, specIcon = GetSpecializationInfoByID(specID)
			local className = UnitClass(self.unit);
			self.titleFrame.specAndClassName:SetText(string.format("%s %s", specName, className));
			SetPortraitToTexture(self.titleFrame.specIcon, specIcon);
			
			-- Set Talent Frame:
			for tier = 1, NUM_TALENTS do
				local button = self.talentFrame["talent"..tier];
				local talentID = ArenaLiveSpectator.UnitCache:GetUnitSelectedTalentByTier(self.unit, tier);
				if ( talentID ) then
					local _, name, icon = GetTalentInfoByID(talentID);
					button.talentID = talentID;
					button.name:SetText(name);
					button.icon:SetTexture(icon);
					button.fade:Hide();
				else
					button.talentID = nil;
					button.name:SetText(L["Not Selected"]);
					button.icon:SetTexture();
					button.fade:Show();
				end
			end
			
			-- Set PVP Talent Frame:
			for tier = 1, NUM_PVPTALENTS do
				local button = self.pvpTalentFrame["pvpTalent"..tier];
				local talentID = ArenaLiveSpectator.UnitCache:GetUnitSelectedPvpTalentByTier(self.unit, tier);
				if ( talentID ) then
					local _, name, icon, available, _, unlocked = GetPvpTalentInfoByID(talentID);
					button.pvpTalentID = talentID;
					button.name:SetText(name);
					button.icon:SetTexture(icon);
					button.fade:Hide();
				else
					button.pvpTalentID = nil;
					button.name:SetText(L["Not Selected"]);
					button.icon:SetTexture();
					button.fade:Show();
				end
			end
		else
			self.waitingOnPlayerInfo = true;
			self:Hide();
		end
	else
		self:Hide();
	end
end

function PlayerInfoFrame:SetUnitAndShow (unit)
	self.unit = unit;
	if ( not self:IsShown() ) then
		self:Show();
	else
		self:Update();
	end
end

function PlayerInfoFrame:OnDragStart ()
	self:StartMoving();
	self:SetClampedToScreen(true);
end

function PlayerInfoFrame:OnShow ()
	self:Update();
end

-- This function is called by ArenaLiveSpectator's OnEvent() method,
-- if "AL_SPEC_PLAYER_SPECIALIZATION_UPDATE" fires.
function PlayerInfoFrame:OnPlayerSpecialisationUpdate(unit)
	if ( self:IsVisible() and unit == self.unit ) then
		if ( self.waitingOnUpdate ) then
			self.waitingOnPlayerInfo = nil;
			self:Show();
		elseif ( self:IsVisible() ) then
			self:Update();
		end
		
	end
end